function plot2DRespPseudo(varargin)
%--------------------------------------------------------------------------
%
% function plot2DRespPseudo(varargin)
%
% This function is called by the menu bar item in
% the figure made from plotOccam2DMT.m.
%
% Function to plot data, response and misfit pseudo sections for OCCAM2DMT
% inversion iterations.  
%
% Version 1.0  Nov, 2006
% Kerry Key
% Scripps Institution of Oceanography
% 
% To do:
%
% Their are some hardwired options below that need to be cleaned up for the
% public release of these codes.  Also need to add colorbar labels and
% boxes to the plots.
% 
%
%--------------------------------------------------------------------------

% Defaults:

   useperiods = 'yes' ;  % 'yes' uses periods, 'no' uses frequency
   usesitepos = 'yes'  ;  % 'yes' uses site position in km, 'no' uses site # for the x scale
   posoffset  = []    ;  % An optional offset (km) to subtract from the site positions to re-zero the x axis  
   lomit      = 'yes' ;  % if 'yes', don't plot data with error  >= 999 (bad data flag)
   
 
% Fixed limits:
%    rholims   = [ -.3 .5];
%   phslims   = [ 0 90];
% or let the code decide what to use:
    rholims =[];
    phslims =[];
    reslims = [-5 5]; % limits for normalized residual plots

%--------------------------------------------------------------------------
% Input the iteration, data and response files:
%--------------------------------------------------------------------------

 % Get the model response file:
 % Actually we get an iteration file to get the data and then look for the
 % corresponding response file:
    if nargin < 1 || ~exist(varargin{1},'file')
        cFileSpec = {'*.iter', 'Iteration files (*.iter)';'*.*','All Files'};
        [fname, pname]=uigetfile( cFileSpec, 'Select Iteration File: ');
        if ~ischar(fname)      % user canceled 
            return
        end
    elseif nargin >= 1
        [pname, fname, sExt] = fileparts( varargin{1} );
        if strcmp(sExt,'.iter'); % make sure its an .iter file
            sExt = '.iter';
        end
        fname = [fname sExt];
    end
    sIterFile = fullfile(pname,fname);
  % Read in the iteration file:  
    [stIter, nParamValues] = readITER( sIterFile  );   
  
  % Read in the data file:
  % Note that we don't actually use the DATA array since it is contained in
  % the RESP array from .resp file.  Here we only use the site and
  % frequency info from the data file.
     [datfrm,datdes,nsites,sites,offsts,nfre,freqs,ndblks,DATA] ...
        = readDATA(stIter.sDataFile);
  
  % Read in the response file:
    [pname, fname, sExt] = fileparts( varargin{1} );
    sRespFile = fullfile(pname,[fname '.resp']);
    if ~exist(sRespFile,'file')  % perhaps we're using the ITER | RESP file pair naming convention...
        sRespFile = sprintf('RESP%2s.resp', fname(end-1:end));
    end
    
    RESP = load( sRespFile );
    
%--------------------------------------------------------------------------
% Decompose the global arrays into local arrays for plotting:
%--------------------------------------------------------------------------

% Initial arrays to nans so that missing data is ok
% D,M,R indicate data, model response and residual:
    DTMr = nan*zeros(nfre+1,nsites+1);  % note the +1 is since pcolor with shading flat doesn't actually plot the last row and column of data.
    DTMp = DTMr;
    DTEr = DTMr;
    DTEp = DTMr;
    RTMr = DTMr;
    RTMp = DTMr;
    RTEr = DTMr;
    RTEp = DTMr;
    MTMr = DTMr;
    MTMp = DTMr;
    MTEr = DTMr;
    MTEp = DTMr;    
  % Append dummy values to freq and site positions to account for pcolor not using last row and column
   freqs = [freqs; freqs(end)];
   offsts = [offsts; offsts(end)];
    
  % RESP array columns 5:7 are the data, response and residual
    for i=1:nsites
        dsite = RESP(RESP(:,1)==i,:);
        esite = DATA(DATA(:,1)==i,5);
        if ~isempty(dsite)
            % TMr data:
             % Get data, omitting if  error >= 999 (flag for omitting data)
            if strcmp(lomit,'yes')
                ii  = dsite(:,3) ==5 & esite <= 999;
            else
                ii  = dsite(:,3) ==5;
            end
            if ~isempty(ii)                
                DTMr(dsite(ii,2),i) = dsite(ii,5);
                 MTMr(dsite(ii,2),i) = dsite(ii,6);
                  RTMr(dsite(ii,2),i) = dsite(ii,7);
            end
            % TMp data:
            if strcmp(lomit,'yes')
                ii  = dsite(:,3) ==6 & esite <= 999;
            else
                ii  = dsite(:,3) ==6;
            end
            if ~isempty(ii)
                DTMp(dsite(ii,2),i) = dsite(ii,5);
                 MTMp(dsite(ii,2),i) = dsite(ii,6);
                  RTMp(dsite(ii,2),i) = dsite(ii,7);
            end
            % TEr data:
            if strcmp(lomit,'yes')
                ii  = dsite(:,3) ==1 & esite <= 999;
            else
                ii  = dsite(:,3) ==1;
            end
            if ~isempty(ii)
                DTEr(dsite(ii,2),i) = dsite(ii,5);
                 MTEr(dsite(ii,2),i) = dsite(ii,6);
                  RTEr(dsite(ii,2),i) = dsite(ii,7);
            end
            % TEp data:
            if strcmp(lomit,'yes')
                ii  = dsite(:,3) ==2 & esite <= 999;
            else
                ii  = dsite(:,3) ==2;
            end
            if ~isempty(ii)
                DTEp(dsite(ii,2),i) = dsite(ii,5);
                 MTEp(dsite(ii,2),i) = dsite(ii,6);
                  RTEp(dsite(ii,2),i) = dsite(ii,7);
            end            
        end
    
    end
  
%--------------------------------------------------------------------------
% Plot the Pseudosections:
%--------------------------------------------------------------------------
 
 % Get the plotting options:
 
  % Are we plotting period or freq?
    if strcmp(useperiods,'yes')
        ypos = log10(1./freqs);
        ylab = 'log_{10} Period (s)';
    else % use frequency
        ypos = log10(freqs);
        ylab = 'log_{10} Frequency (Hz)'; 
    end
 % Are we plotting site position in distance or site #?
    if strcmp(usesitepos,'yes')
        xpos = offsts/1000;
        xlab = 'Position (km)';
        % Check for a distance offset to add to plot:
        if ~isempty(posoffset)
            xpos = xpos + posoffset;
        end
    else % use frequency
        xpos = 1:nsites+1;  %+1 is dummy value since pcolor omits last row and column
        xlab = 'Site #'; 
    end

    
 % Now make the figures:
    mtit = 'Model';
    rtit = 'Residual';
    
  % TM data, response, misfit  
    if ~all(all(isnan(DTMr))) & ~all(all(isnan(DTMp)))
        dtit = 'TM Data';
        plot2Dpseudo2x3cols(xpos,ypos,DTMr,DTMp,MTMr,MTMp,RTMr,RTMp,...
                            xlab,ylab,dtit,mtit,rtit,rholims,phslims,reslims)
    end
    
  % TE data, response, misfit  
    if ~all(all(isnan(DTEr))) & ~all(all(isnan(DTEp)))
        dtit = 'TE Data';
        plot2Dpseudo2x3cols(xpos,ypos,DTEr,DTEp,MTEr,MTEp,RTEr,RTEp,...
                            xlab,ylab,dtit,mtit,rtit,rholims,phslims,reslims)
    end    
        
%--------------------------------------------------------------------------
function plot2Dpseudo2x3cols(xpos,ypos,DTMr,DTMp,MTMr,MTMp,RTMr,RTMp,...
                             xlab,ylab,dtit,mtit,rtit,rholims,phslims,reslims)
%
% Subfunction that does common plotting of some arrays
% Creates a 2 row by 3 column plot of apparent resistivity and phase
% pseudosections for data,model response and misfit
%
%--------------------------------------------------------------------------        
        
        hfig = figure;
        % flip colorscale to make blue resistive;
        colormap(flipud(colormap));
        
     % Apparent resistivity:
     
        % Data
        hd = subplot(2,3,1);
        pcolor(xpos,ypos,DTMr);
        axis ij;
        shading flat;
        xlabel(xlab)
        ylabel(ylab)
        title(dtit)
        colorbar;
        
        % Model Response:
        hm = subplot(2,3,2);
        pcolor(xpos,ypos,MTMr);
        axis ij;
           box on;
        shading flat;
        xlabel(xlab)
        ylabel(ylab)
        title(mtit)
        colorbar;
       
        % Adjust data and model response caxis limits to be same:
        if isempty(rholims); % auto compute limits
          axis(hd); dlims = caxis;
          axis(hm); mlims = caxis;
          lims = [min([dlims(1) mlims(1)]) max([dlims(2) mlims(2)]) ];
          axes(hd);caxis(lims);colorbar
          axes(hm);caxis(lims);colorbar
        else
          axes(hd);caxis(rholims);colorbar
          axes(hm);caxis(rholims);colorbar   
        end
        
        % Residual
        hr = subplot(2,3,3);
        pcolor(xpos,ypos,RTMr);
        axis ij;
        shading flat;
           box on;
        xlabel(xlab)
        ylabel(ylab)
        title(rtit)
        if ~isempty(reslims)
            caxis(reslims);
        end
        colorbar;
        
      % Phase
        % Data
        hd = subplot(2,3,4);
        pcolor(xpos,ypos,DTMp);
        axis ij;
        shading flat;
        box on;
        xlabel(xlab)
        ylabel(ylab)
        title('Phase')
        colorbar;
        
        % Model Response:
        hm = subplot(2,3,5);
        pcolor(xpos,ypos,MTMp);
        axis ij;
        shading flat;
           box on;
        xlabel(xlab)
        ylabel(ylab)
        colorbar;
  
        % Adjust data and model response caxis limits to be same:
        if isempty(phslims); % auto compute limits
          axis(hd); dlims = caxis;
          axis(hm); mlims = caxis;
          lims = [min([dlims(1) mlims(1)]) max([dlims(2) mlims(2)]) ];
          axes(hd);caxis(lims);colorbar
          axes(hm);caxis(lims);colorbar
        else
          axes(hd);caxis(phslims);colorbar
          axes(hm);caxis(phslims);colorbar   
        end
        
        % Residual
        hr = subplot(2,3,6);
        pcolor(xpos,ypos,RTMp);
        axis ij;
        shading flat;
        xlabel(xlab)
        ylabel(ylab)
        if ~isempty(reslims)
            caxis(reslims);
        end
        colorbar;
        
        
         
%--------------------------------------------------------------------------
function [datfrm,datdes,nsites,sites,offsts,nfre,freqs, ...
        ndblks,DATA]= readDATA(datfile)
%--------------------------------------------------------------------------
% function readDATA.m
% Version 1.0, July 2001
% Written by: Kerry Key IGPP/SIO/UCSD
% 
% A matlab script to read an OCCAM2DMT DATA file
%
% Usage: 
%        [datfrm,datdes,nsites,sites,offsts,nfre,freqs, ...
%         ndblks,DATA]= readDATA(datfile);
%
% Inputs
%        datfile: OCCAM2DMT DATA file name (string format)
%
% Outputs: 
% 		datfrm
% 		datdes
% 		nsites
% 		sites(nsites)
% 		offsts(nsites)
% 		nfre:            Also can be found in MESH FILE, if running FWD PW2D 
% 		freqs(nfre):     Also can be found in  MESH FILE, if running FWD PW2D
% 		ndblks:          Number of data blocks
% 		DATA(ndblks,5):  Ndblks by [SITE  FREQ#  TYPE  DATUM  ERR]
%
%--------------------------------------------------------------------------


% OPEN AND READ IN DATA FILE (to get site offsets):
   fiddat = fopen(datfile,'r');
    cline  = fgets(fiddat);  
   datfrm = sscanf(cline,'FORMAT:           %80c');
   if  strcmp(deblank(datfrm),'OCCAM2DMTDATA_2.0')
      ndp = 4;
   else
       ndp =3;
   end
   
    cline  = fgets(fiddat); 
   datdes = sscanf(cline,'DESCRIPTION:      %80c');
    cline  = fgets(fiddat); 
   nsites = sscanf(cline,'SITES:            %f');
   for i  = 1:nsites
       cline    = fgets(fiddat);
      sites(i) = cellstr(sscanf(cline,'%s'));
   end
   string = fgets(fiddat);   %#ok<NASGU>
   offsts = fscanf(fiddat,'%f',nsites);
    cline  = fgets(fiddat);  %#ok<NASGU>
    cline  = fgetl(fiddat); 
   nfre   = sscanf(cline,'FREQUENCIES:%f');
   freqs  = fscanf(fiddat,'%f',nfre);
    cline  = fgets(fiddat); %#ok<NASGU>
    cline  = fgets(fiddat);
   ndblks = sscanf(cline,'DATA BLOCKS:%f');
    string = fgets(fiddat); %#ok<NASGU>
   DATA   = fscanf(fiddat,'%f %f %f %f %f',[ndp+2 ndblks]);
   DATA=DATA';
   fclose(fiddat);    
    
%--------------------------------------------------------------------------    
function [stIter, nParams] = readITER( sFile )
%--------------------------------------------------------------------------
% readITER - read & parse an Occam2D iteration file.
%
% v1.0, July 2001 by: Kerry Key IGPP/SIO/UCSD
% v2.0, Nov 2006 by David Myer - rewritten to support new OCCAMITER_FLEX
% format files which have new entries for diagonal penalties, and roughness
% modification.  And which no longer require entries to be in a particular
% order.
%
% Parameters:
%   sFile   - path+name of iteration file to open.
% Returns:
%   stIter  - structure with members for each of the header items.  See code
%           for a list of possible members.  Structure will only have members
%           for the items that exist in the iteration file.  So not all
%           members will be present every time.
%   nParams - parameter values
%--------------------------------------------------------------------------

    % Open the iteration file
    fid = fopen( sFile, 'r' );
    
    % Default a value or two that should always exist.
    stIter.nMisfit      = 1000.0;
    stIter.nRoughness   = 0.0;
    
    % The header of the file consists of a bunch of "code: value" pairs
    % terminated by a pair that tells how many parameters there are (and is
    % immediately followed by the parameter list).
    % Read & process the pairs.
    while ~ feof(fid)
        % Get the current line & break up the code / value pair
        sLine = fgets( fid );
        [sCode, sValue] = strtok( sLine, ':' );
        sCode = lower(strtrim(sCode));
        sValue(1) = [];     % remove the leading token
        
        % If there is a user comment in the value, eliminate it.
        sValue = strtrim( strtok(sValue, '!%') );
        
        % Which code do we have?
        switch (sCode)
        case {'format'}
            stIter.sFormat = sValue;
            
        case {'description'}
            stIter.sDesc = sValue;
            
        case {'model file'}
            stIter.sModelFile = sValue;
            
        case {'data file'}
            stIter.sDataFile = sValue;
            
        case {'date/time'}
            stIter.sDateTime = sValue;
            
        case {'max iter', 'iterations to run'}
            stIter.nIterToRun = str2double( sValue );
            
        case {'req tol', 'target misfit'}
            stIter.nTargetMisfit = str2double( sValue );
            
        case {'iruf', 'roughness type'}
            stIter.nRoughnessType = str2double( sValue );
            stIter.bDiagPenalties = (stIter.nRoughnessType < 0);
            stIter.nRoughnessType = abs( stIter.nRoughnessType );
            
        case {'diagonal penalties'}
            stIter.bDiagPenalties = (str2double( sValue ) ~= 0);
            if isfield( stIter, 'nRoughnessType' )
                stIter.nRoughnessType = abs( stIter.nRoughnessType );
            end
            
        case {'debug level'}
            stIter.DebugLevel = str2double( sValue );
            
        case {'iteration'}
            stIter.nIteration = str2double( sValue );
            
        case {'pmu', 'lagrange value'}
            stIter.nLagrange = str2double( sValue );
            
        case {'rlast', 'roughness value'}
            stIter.nRoughness = str2double( sValue );
            
        case {'tlast', 'misfit value'}
            stIter.nMisfit = str2double( sValue );
            
        case {'ifftol', 'misfit reached'}
            stIter.bReachedMisfit = (str2double( sValue ) ~= 0);
            
        case {'modify roughness'}
            stIter.sModifyRoughness = sValue;
            
        case {'no. parms', 'param count'}
            stIter.nParams = str2double( sValue );
            break;	% The next thing in the file is the data param list!
            
            otherwise % do nothing
                
%         case default
%             disp( 'Error reading startup/iteration file!' );
%             error( ['Unknown or unsupported code: ' sCode] );
            
        end
        
    end
    if feof(fid) 
        error( ['Error: iteration file ' sFile ' terminated early!'] );
    end
    
    % Read the parameters
    nParams = fscanf( fid, '%f', stIter.nParams );
    
    % Close up & return
    fclose(fid);

    return   
 